@@ -2593,6 +2593,7 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
 {
   DBusString directory;
   const char *tmp;
+  dbus_bool_t ret = FALSE;
 
   if (!_dbus_string_init (&directory))
     return FALSE;
@@ -2600,16 +2601,14 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
   tmp = _dbus_get_tmpdir ();
 
   if (tmp == NULL)
-    return FALSE;
+    goto out;
 
   if (!_dbus_string_append (&directory, tmp))
-    return FALSE;
+    goto out;
 
   if (!_dbus_string_append (&directory, "/dbus-reload-test-") ||
       !_dbus_generate_random_ascii (&directory, 6, NULL))
-     {
-       return FALSE;
-     }
+    goto out;
 
   /* Do normal tests */
   if (!init_service_reload_test (&directory))
@@ -2631,11 +2630,13 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
 
   /* Cleanup test directory */
   if (!cleanup_service_reload_test (&directory))
-    return FALSE;
+    goto out;
 
+  ret = TRUE;
+
+out:
   _dbus_string_free (&directory);
-
-  return TRUE;
+  return ret;
 }
 
 #endif /* DBUS_ENABLE_EMBEDDED_TESTS */
