@@ -35,9 +35,11 @@
 
 DBUS_BEGIN_DECLS
 
+DBUS_PRIVATE_EXPORT
 void _dbus_warn               (const char *format,
                                ...) _DBUS_GNUC_PRINTF (1, 2);
 
+DBUS_PRIVATE_EXPORT
 void _dbus_warn_check_failed  (const char *format,
                                ...) _DBUS_GNUC_PRINTF (1, 2);
 
@@ -89,16 +91,24 @@ void _dbus_warn_check_failed  (const char *format,
 #endif
 
 #ifdef DBUS_CPP_SUPPORTS_VARIABLE_MACRO_ARGUMENTS
+DBUS_PRIVATE_EXPORT
 void _dbus_verbose_real       (const char *file, const int line, const char *function, 
                                const char *format,...) _DBUS_GNUC_PRINTF (4, 5);
 #  define _dbus_verbose(fmt,...) _dbus_verbose_real( __FILE__,__LINE__,__FUNCTION__,fmt, ## __VA_ARGS__)
 #else
+DBUS_PRIVATE_EXPORT
 void _dbus_verbose_real       (const char *format,
                                ...) _DBUS_GNUC_PRINTF (1, 2);
 #  define _dbus_verbose _dbus_verbose_real
 #endif
+DBUS_PRIVATE_EXPORT
 void _dbus_verbose_reset_real (void);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_is_verbose_real (void);
+DBUS_PRIVATE_EXPORT
+dbus_bool_t _dbus_get_verbose (void);
+DBUS_PRIVATE_EXPORT
+void _dbus_set_verbose (dbus_bool_t state);
 
 #  define _dbus_verbose_reset _dbus_verbose_reset_real
 #  define _dbus_is_verbose _dbus_is_verbose_real
@@ -122,11 +132,13 @@ void _dbus_trace_ref (const char *obj_name,
                       const char *env_var,
                       int        *enabled);
 
+DBUS_PRIVATE_EXPORT
 const char* _dbus_strerror (int error_number);
 
 #ifdef DBUS_DISABLE_ASSERT
 #define _dbus_assert(condition) do { } while (0)
 #else
+DBUS_PRIVATE_EXPORT
 void _dbus_real_assert (dbus_bool_t  condition,
                         const char  *condition_text,
                         const char  *file,
@@ -139,6 +151,7 @@ void _dbus_real_assert (dbus_bool_t  condition,
 #ifdef DBUS_DISABLE_ASSERT
 #define _dbus_assert_not_reached(explanation) do { } while (0)
 #else
+DBUS_PRIVATE_EXPORT
 void _dbus_real_assert_not_reached (const char *explanation,
                                     const char *file,
                                     int         line) _DBUS_GNUC_NORETURN;
@@ -151,6 +164,7 @@ void _dbus_real_assert_not_reached (const char *explanation,
 #define _dbus_return_val_if_fail(condition, val)
 #else
 
+DBUS_PRIVATE_EXPORT
 extern const char *_dbus_return_if_fail_warning_format;
 
 #define _dbus_return_if_fail(condition) do {                                       \
@@ -188,8 +202,20 @@ extern const char *_dbus_return_if_fail_warning_format;
 #define _DBUS_ASSERT_ERROR_IS_SET(error) do { } while (0)
 #define _DBUS_ASSERT_ERROR_IS_CLEAR(error) do { } while (0)
 #else
-#define _DBUS_ASSERT_ERROR_IS_SET(error)   _dbus_assert ((error) == NULL || dbus_error_is_set ((error)))
-#define _DBUS_ASSERT_ERROR_IS_CLEAR(error) _dbus_assert ((error) == NULL || !dbus_error_is_set ((error)))
+static inline void
+_dbus_assert_error_is_set (const DBusError *error)
+{
+    _dbus_assert (error == NULL || dbus_error_is_set (error));
+}
+
+static inline void
+_dbus_assert_error_is_clear (const DBusError *error)
+{
+    _dbus_assert (error == NULL || !dbus_error_is_set (error));
+}
+
+#define _DBUS_ASSERT_ERROR_IS_SET(error) _dbus_assert_error_is_set(error)
+#define _DBUS_ASSERT_ERROR_IS_CLEAR(error) _dbus_assert_error_is_clear(error)
 #endif
 
 #define _dbus_return_if_error_is_set(error) _dbus_return_if_fail ((error) == NULL || !dbus_error_is_set ((error)))
@@ -213,9 +239,11 @@ extern const char *_dbus_return_if_fail_warning_format;
   ((void*)_DBUS_ALIGN_VALUE(this, boundary))
 
 
+DBUS_PRIVATE_EXPORT
 char*       _dbus_strdup                (const char  *str);
 void*       _dbus_memdup                (const void  *mem,
                                          size_t       n_bytes);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_string_array_contains (const char **array,
                                          const char  *str);
 char**      _dbus_dup_string_array      (const char **array);
@@ -251,18 +279,18 @@ char**      _dbus_dup_string_array      (const char **array);
 typedef void (* DBusForeachFunction) (void *element,
                                       void *data);
 
-dbus_bool_t _dbus_set_fd_nonblocking (int             fd,
-                                      DBusError      *error);
-
 void _dbus_verbose_bytes           (const unsigned char *data,
                                     int                  len,
                                     int                  offset);
+DBUS_PRIVATE_EXPORT
 void _dbus_verbose_bytes_of_string (const DBusString    *str,
                                     int                  start,
                                     int                  len);
 
+DBUS_PRIVATE_EXPORT
 extern const char *_dbus_no_memory_message;
 #define _DBUS_SET_OOM(error) dbus_set_error_const ((error), DBUS_ERROR_NO_MEMORY, _dbus_no_memory_message)
+DBUS_PRIVATE_EXPORT
 void _dbus_set_error_valist (DBusError  *error,
                              const char *name,
                              const char *format,
@@ -276,9 +304,11 @@ void        _dbus_set_fail_alloc_failures       (int  failures_per_failure);
 int         _dbus_get_fail_alloc_failures       (void);
 dbus_bool_t _dbus_decrement_fail_alloc_counter  (void);
 dbus_bool_t _dbus_disable_mem_pools             (void);
+DBUS_PRIVATE_EXPORT
 int         _dbus_get_malloc_blocks_outstanding (void);
 
 typedef dbus_bool_t (* DBusTestMemoryFunction)  (void *data);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_test_oom_handling (const char             *description,
                                      DBusTestMemoryFunction  func,
                                      void                   *data);
@@ -295,6 +325,7 @@ dbus_bool_t _dbus_test_oom_handling (const char             *description,
 #endif /* !DBUS_ENABLE_EMBEDDED_TESTS */
 
 typedef void (* DBusShutdownFunction) (void *data);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_register_shutdown_func          (DBusShutdownFunction  function,
                                                    void                 *data);
 dbus_bool_t _dbus_register_shutdown_func_unlocked (DBusShutdownFunction  function,
@@ -333,6 +364,7 @@ void        _dbus_unlock (DBusGlobalLock lock);
 #define _DBUS_LOCK(name)                _dbus_lock   (_DBUS_LOCK_##name)
 #define _DBUS_UNLOCK(name)              _dbus_unlock (_DBUS_LOCK_##name)
 
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_threads_init_debug (void);
 
 dbus_bool_t   _dbus_address_append_escaped (DBusString       *escaped,
@@ -357,7 +389,10 @@ union DBusGUID
   char as_bytes[DBUS_UUID_LENGTH_BYTES];                /**< guid as 16 single-byte values */
 };
 
-void        _dbus_generate_uuid  (DBusGUID         *uuid);
+DBUS_PRIVATE_EXPORT _DBUS_GNUC_WARN_UNUSED_RESULT
+dbus_bool_t _dbus_generate_uuid  (DBusGUID         *uuid,
+                                  DBusError        *error);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_uuid_encode    (const DBusGUID   *uuid,
                                   DBusString       *encoded);
 dbus_bool_t _dbus_read_uuid_file (const DBusString *filename,
@@ -369,7 +404,8 @@ dbus_bool_t _dbus_write_uuid_file (const DBusString *filename,
                                    const DBusGUID   *uuid,
                                    DBusError        *error);
 
-dbus_bool_t _dbus_get_local_machine_uuid_encoded (DBusString *uuid_str);
+dbus_bool_t _dbus_get_local_machine_uuid_encoded (DBusString *uuid_str,
+                                                  DBusError  *error);
 
 #define _DBUS_PASTE2(a, b) a ## b
 #define _DBUS_PASTE(a, b) _DBUS_PASTE2 (a, b)
