@@ -49,7 +49,7 @@ typedef struct DBusTransportSocket DBusTransportSocket;
 struct DBusTransportSocket
 {
   DBusTransport base;                   /**< Parent instance */
-  int fd;                               /**< File descriptor. */
+  DBusSocket fd;                        /**< File descriptor. */
   DBusWatch *read_watch;                /**< Watch for readability. */
   DBusWatch *write_watch;               /**< Watch for writability. */
 
@@ -159,9 +159,9 @@ check_write_watch (DBusTransport *transport)
         }
     }
 
-  _dbus_verbose ("check_write_watch(): needed = %d on connection %p watch %p fd = %d outgoing messages exist %d\n",
+  _dbus_verbose ("check_write_watch(): needed = %d on connection %p watch %p fd = %" DBUS_SOCKET_FORMAT " outgoing messages exist %d\n",
                  needed, transport->connection, socket_transport->write_watch,
-                 socket_transport->fd,
+                 _dbus_socket_printable (socket_transport->fd),
                  _dbus_connection_has_messages_to_send_unlocked (transport->connection));
 
   _dbus_connection_toggle_watch_unlocked (transport->connection,
@@ -177,7 +177,8 @@ check_read_watch (DBusTransport *transport)
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
   dbus_bool_t need_read_watch;
 
-  _dbus_verbose ("fd = %d\n",socket_transport->fd);
+  _dbus_verbose ("fd = %" DBUS_SOCKET_FORMAT "\n",
+                 _dbus_socket_printable (socket_transport->fd));
   
   if (transport->connection == NULL)
     return;
@@ -247,13 +248,15 @@ read_data_into_auth (DBusTransport *transport,
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
   DBusString *buffer;
   int bytes_read;
-  
+  int saved_errno;
+
   *oom = FALSE;
 
   _dbus_auth_get_buffer (transport->auth, &buffer);
   
   bytes_read = _dbus_read_socket (socket_transport->fd,
                                   buffer, socket_transport->max_bytes_read_per_iteration);
+  saved_errno = _dbus_save_socket_errno ();
 
   _dbus_auth_return_buffer (transport->auth, buffer);
 
@@ -267,16 +270,16 @@ read_data_into_auth (DBusTransport *transport,
     {
       /* EINTR already handled for us */
 
-      if (_dbus_get_is_errno_enomem ())
+      if (_dbus_get_is_errno_enomem (saved_errno))
         {
           *oom = TRUE;
         }
-      else if (_dbus_get_is_errno_eagain_or_ewouldblock ())
+      else if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno))
         ; /* do nothing, just return FALSE below */
       else
         {
           _dbus_verbose ("Error reading from remote app: %s\n",
-                         _dbus_strerror_from_errno ());
+                         _dbus_strerror (saved_errno));
           do_io_error (transport);
         }
 
@@ -299,6 +302,7 @@ write_data_from_auth (DBusTransport *transport)
 {
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
   int bytes_written;
+  int saved_errno;
   const DBusString *buffer;
 
   if (!_dbus_auth_get_bytes_to_send (transport->auth,
@@ -308,6 +312,7 @@ write_data_from_auth (DBusTransport *transport)
   bytes_written = _dbus_write_socket (socket_transport->fd,
                                       buffer,
                                       0, _dbus_string_get_length (buffer));
+  saved_errno = _dbus_save_socket_errno ();
 
   if (bytes_written > 0)
     {
@@ -318,12 +323,12 @@ write_data_from_auth (DBusTransport *transport)
     {
       /* EINTR already handled for us */
       
-      if (_dbus_get_is_errno_eagain_or_ewouldblock ())
+      if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno))
         ;
       else
         {
           _dbus_verbose ("Error writing to remote app: %s\n",
-                         _dbus_strerror_from_errno ());
+                         _dbus_strerror (saved_errno));
           do_io_error (transport);
         }
     }
@@ -510,9 +515,9 @@ do_writing (DBusTransport *transport)
     }
 
 #if 1
-  _dbus_verbose ("do_writing(), have_messages = %d, fd = %d\n",
+  _dbus_verbose ("do_writing(), have_messages = %d, fd = %" DBUS_SOCKET_FORMAT "\n",
                  _dbus_connection_has_messages_to_send_unlocked (transport->connection),
-                 socket_transport->fd);
+                 _dbus_socket_printable (socket_transport->fd));
 #endif
   
   oom = FALSE;
@@ -527,6 +532,7 @@ do_writing (DBusTransport *transport)
       const DBusString *body;
       int header_len, body_len;
       int total_bytes_to_write;
+      int saved_errno;
       
       if (total > socket_transport->max_bytes_written_per_iteration)
         {
@@ -584,6 +590,7 @@ do_writing (DBusTransport *transport)
                                 &socket_transport->encoded_outgoing,
                                 socket_transport->message_bytes_written,
                                 total_bytes_to_write - socket_transport->message_bytes_written);
+          saved_errno = _dbus_save_socket_errno ();
         }
       else
         {
@@ -612,6 +619,7 @@ do_writing (DBusTransport *transport)
                                                       0, body_len,
                                                       unix_fds,
                                                       n);
+              saved_errno = _dbus_save_socket_errno ();
 
               if (bytes_written > 0 && n > 0)
                 _dbus_verbose("Wrote %i unix fds\n", n);
@@ -638,6 +646,8 @@ do_writing (DBusTransport *transport)
                                         body_len -
                                         (socket_transport->message_bytes_written - header_len));
                 }
+
+              saved_errno = _dbus_save_socket_errno ();
             }
         }
 
@@ -645,16 +655,48 @@ do_writing (DBusTransport *transport)
         {
           /* EINTR already handled for us */
           
-          /* For some discussion of why we also ignore EPIPE here, see
+          /* If the other end closed the socket with close() or shutdown(), we
+           * receive EPIPE here but we must not close the socket yet: there
+           * might still be some data to read. See:
            * http://lists.freedesktop.org/archives/dbus/2008-March/009526.html
            */
           
-          if (_dbus_get_is_errno_eagain_or_ewouldblock () || _dbus_get_is_errno_epipe ())
+          if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno) || _dbus_get_is_errno_epipe (saved_errno))
             goto out;
+
+          /* Since Linux commit 25888e (from 2.6.37-rc4, Nov 2010), sendmsg()
+           * on Unix sockets returns -1 errno=ETOOMANYREFS when the passfd
+           * mechanism (SCM_RIGHTS) is used recursively with a recursion level
+           * of maximum 4. The kernel does not have an API to check whether
+           * the passed fds can be forwarded and it can change asynchronously.
+           * See:
+           * https://bugs.freedesktop.org/show_bug.cgi?id=80163
+           */
+
+          else if (_dbus_get_is_errno_etoomanyrefs (saved_errno))
+            {
+              /* We only send fds in the first byte of the message.
+               * ETOOMANYREFS cannot happen after.
+               */
+              _dbus_assert (socket_transport->message_bytes_written == 0);
+
+              _dbus_verbose (" discard message of %d bytes due to ETOOMANYREFS\n",
+                             total_bytes_to_write);
+
+              socket_transport->message_bytes_written = 0;
+              _dbus_string_set_length (&socket_transport->encoded_outgoing, 0);
+              _dbus_string_compact (&socket_transport->encoded_outgoing, 2048);
+
+              /* The message was not actually sent but it needs to be removed
+               * from the outgoing queue
+               */
+              _dbus_connection_message_sent_unlocked (transport->connection,
+                                                      message);
+            }
           else
             {
               _dbus_verbose ("Error writing to remote app: %s\n",
-                             _dbus_strerror_from_errno ());
+                             _dbus_strerror (saved_errno));
               do_io_error (transport);
               goto out;
             }
@@ -698,8 +740,10 @@ do_reading (DBusTransport *transport)
   int bytes_read;
   int total;
   dbus_bool_t oom;
+  int saved_errno;
 
-  _dbus_verbose ("fd = %d\n",socket_transport->fd);
+  _dbus_verbose ("fd = %" DBUS_SOCKET_FORMAT "\n",
+                 _dbus_socket_printable (socket_transport->fd));
   
   /* No messages without authentication! */
   if (!_dbus_transport_try_to_authenticate (transport))
@@ -742,6 +786,8 @@ do_reading (DBusTransport *transport)
                                         &socket_transport->encoded_incoming,
                                         socket_transport->max_bytes_read_per_iteration);
 
+      saved_errno = _dbus_save_socket_errno ();
+
       _dbus_assert (_dbus_string_get_length (&socket_transport->encoded_incoming) ==
                     bytes_read);
       
@@ -791,6 +837,7 @@ do_reading (DBusTransport *transport)
                                                        buffer,
                                                        socket_transport->max_bytes_read_per_iteration,
                                                        fds, &n_fds);
+          saved_errno = _dbus_save_socket_errno ();
 
           if (bytes_read >= 0 && n_fds > 0)
             _dbus_verbose("Read %i unix fds\n", n_fds);
@@ -802,28 +849,29 @@ do_reading (DBusTransport *transport)
         {
           bytes_read = _dbus_read_socket (socket_transport->fd,
                                           buffer, socket_transport->max_bytes_read_per_iteration);
+          saved_errno = _dbus_save_socket_errno ();
         }
 
       _dbus_message_loader_return_buffer (transport->loader,
                                           buffer);
     }
-  
+
   if (bytes_read < 0)
     {
       /* EINTR already handled for us */
 
-      if (_dbus_get_is_errno_enomem ())
+      if (_dbus_get_is_errno_enomem (saved_errno))
         {
           _dbus_verbose ("Out of memory in read()/do_reading()\n");
           oom = TRUE;
           goto out;
         }
-      else if (_dbus_get_is_errno_eagain_or_ewouldblock ())
+      else if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno))
         goto out;
       else
         {
           _dbus_verbose ("Error reading from remote app: %s\n",
-                         _dbus_strerror_from_errno ());
+                         _dbus_strerror (saved_errno));
           do_io_error (transport);
           goto out;
         }
@@ -961,7 +1009,7 @@ socket_handle_watch (DBusTransport *transport,
         _dbus_verbose ("asked to handle write watch with non-write condition 0x%x\n",
                        flags);
       else
-        _dbus_verbose ("asked to handle watch %p on fd %d that we don't recognize\n",
+        _dbus_verbose ("asked to handle watch %p on fd %" DBUS_SOCKET_FORMAT " that we don't recognize\n",
                        watch, dbus_watch_get_socket (watch));
     }
 #endif /* DBUS_ENABLE_VERBOSE_MODE */
@@ -979,7 +1027,7 @@ socket_disconnect (DBusTransport *transport)
   free_watches (transport);
   
   _dbus_close_socket (socket_transport->fd, NULL);
-  socket_transport->fd = -1;
+  _dbus_socket_invalidate (&socket_transport->fd);
 }
 
 static dbus_bool_t
@@ -1030,13 +1078,13 @@ socket_do_iteration (DBusTransport *transport,
   int poll_res;
   int poll_timeout;
 
-  _dbus_verbose (" iteration flags = %s%s timeout = %d read_watch = %p write_watch = %p fd = %d\n",
+  _dbus_verbose (" iteration flags = %s%s timeout = %d read_watch = %p write_watch = %p fd = %" DBUS_SOCKET_FORMAT "\n",
                  flags & DBUS_ITERATION_DO_READING ? "read" : "",
                  flags & DBUS_ITERATION_DO_WRITING ? "write" : "",
                  timeout_milliseconds,
                  socket_transport->read_watch,
                  socket_transport->write_watch,
-                 socket_transport->fd);
+                 _dbus_socket_printable (socket_transport->fd));
   
   /* the passed in DO_READING/DO_WRITING flags indicate whether to
    * read/write messages, but regardless of those we may need to block
@@ -1044,7 +1092,7 @@ socket_do_iteration (DBusTransport *transport,
    * we don't want to read any messages yet if not given DO_READING.
    */
 
-  poll_fd.fd = socket_transport->fd;
+  poll_fd.fd = _dbus_socket_get_pollable (socket_transport->fd);
   poll_fd.events = 0;
   
   if (_dbus_transport_try_to_authenticate (transport))
@@ -1097,6 +1145,8 @@ socket_do_iteration (DBusTransport *transport,
 
   if (poll_fd.events)
     {
+      int saved_errno;
+
       if (flags & DBUS_ITERATION_BLOCK)
 	poll_timeout = timeout_milliseconds;
       else
@@ -1115,8 +1165,9 @@ socket_do_iteration (DBusTransport *transport,
       
     again:
       poll_res = _dbus_poll (&poll_fd, 1, poll_timeout);
+      saved_errno = _dbus_save_socket_errno ();
 
-      if (poll_res < 0 && _dbus_get_is_errno_eintr ())
+      if (poll_res < 0 && _dbus_get_is_errno_eintr (saved_errno))
 	goto again;
 
       if (flags & DBUS_ITERATION_BLOCK)
@@ -1159,7 +1210,7 @@ socket_do_iteration (DBusTransport *transport,
       else
         {
           _dbus_verbose ("Error from _dbus_poll(): %s\n",
-                         _dbus_strerror_from_errno ());
+                         _dbus_strerror (saved_errno));
         }
     }
 
@@ -1190,7 +1241,7 @@ socket_live_messages_changed (DBusTransport *transport)
 
 static dbus_bool_t
 socket_get_socket_fd (DBusTransport *transport,
-                      int           *fd_p)
+                      DBusSocket    *fd_p)
 {
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
   
@@ -1221,7 +1272,7 @@ static const DBusTransportVTable socket_vtable = {
  * @returns the new transport, or #NULL if no memory.
  */
 DBusTransport*
-_dbus_transport_new_for_socket (int               fd,
+_dbus_transport_new_for_socket (DBusSocket        fd,
                                 const DBusString *server_guid,
                                 const DBusString *address)
 {
@@ -1237,14 +1288,14 @@ _dbus_transport_new_for_socket (int               fd,
   if (!_dbus_string_init (&socket_transport->encoded_incoming))
     goto failed_1;
   
-  socket_transport->write_watch = _dbus_watch_new (fd,
+  socket_transport->write_watch = _dbus_watch_new (_dbus_socket_get_pollable (fd),
                                                  DBUS_WATCH_WRITABLE,
                                                  FALSE,
                                                  NULL, NULL, NULL);
   if (socket_transport->write_watch == NULL)
     goto failed_2;
   
-  socket_transport->read_watch = _dbus_watch_new (fd,
+  socket_transport->read_watch = _dbus_watch_new (_dbus_socket_get_pollable (fd),
                                                 DBUS_WATCH_READABLE,
                                                 FALSE,
                                                 NULL, NULL, NULL);
@@ -1302,7 +1353,7 @@ _dbus_transport_new_for_tcp_socket (const char     *host,
                                     const char     *noncefile,
                                     DBusError      *error)
 {
-  int fd;
+  DBusSocket fd;
   DBusTransport *transport;
   DBusString address;
   
@@ -1339,7 +1390,7 @@ _dbus_transport_new_for_tcp_socket (const char     *host,
     goto error;
 
   fd = _dbus_connect_tcp_socket_with_nonce (host, port, family, noncefile, error);
-  if (fd < 0)
+  if (!_dbus_socket_is_valid (fd))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
       _dbus_string_free (&address);
@@ -1355,7 +1406,7 @@ _dbus_transport_new_for_tcp_socket (const char     *host,
     {
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
       _dbus_close_socket (fd, NULL);
-      fd = -1;
+      _dbus_socket_invalidate (&fd);
     }
 
   return transport;
