@@ -128,11 +128,13 @@ static Config no_eavesdrop_config = {
     FALSE
 };
 
+#ifdef DBUS_UNIX
 static Config fake_systemd_config = {
     "valid-config-files/systemd-activation.conf",
     NULL,
     FALSE
 };
+#endif
 
 static Config side_effects_config = {
     NULL,
@@ -164,22 +166,22 @@ _log_message (DBusMessage *m,
     const char *file,
     int line)
 {
-  g_message ("%s:%d: message type %d (%s)", file, line,
+  g_test_message ("%s:%d: message type %d (%s)", file, line,
       dbus_message_get_type (m),
       dbus_message_type_to_string (dbus_message_get_type (m)));
-  g_message ("\tfrom: %s",
+  g_test_message ("\tfrom: %s",
       not_null2 (dbus_message_get_sender (m), "(dbus-daemon)"));
-  g_message ("\tto: %s",
+  g_test_message ("\tto: %s",
       not_null2 (dbus_message_get_destination (m), "(broadcast)"));
-  g_message ("\tpath: %s",
+  g_test_message ("\tpath: %s",
       not_null (dbus_message_get_path (m)));
-  g_message ("\tinterface: %s",
+  g_test_message ("\tinterface: %s",
       not_null (dbus_message_get_interface (m)));
-  g_message ("\tmember: %s",
+  g_test_message ("\tmember: %s",
       not_null (dbus_message_get_member (m)));
-  g_message ("\tsignature: %s",
+  g_test_message ("\tsignature: %s",
       not_null (dbus_message_get_signature (m)));
-  g_message ("\terror name: %s",
+  g_test_message ("\terror name: %s",
       not_null (dbus_message_get_error_name (m)));
 
   if (strcmp ("s", dbus_message_get_signature (m)) == 0)
@@ -191,7 +193,7 @@ _log_message (DBusMessage *m,
             DBUS_TYPE_STRING, &s,
             DBUS_TYPE_INVALID);
       test_assert_no_error (&e);
-      g_message ("\tstring payload: %s", s);
+      g_test_message ("\tstring payload: %s", s);
     }
 }
 
@@ -454,6 +456,8 @@ become_monitor (Fixture *f)
   int i;
   dbus_uint32_t zero = 0;
 
+  dbus_connection_set_route_peer_messages (f->monitor, TRUE);
+
   if (f->config != NULL && f->config->match_rules != NULL)
     match_rules = f->config->match_rules;
   else
@@ -509,6 +513,155 @@ become_monitor (Fixture *f)
   m = NULL;
 }
 
+/*
+ * Test what happens if the method call arguments are invalid.
+ */
+static void
+test_invalid (Fixture *f,
+    gconstpointer context)
+{
+  DBusMessage *m;
+  DBusPendingCall *pc;
+  dbus_bool_t ok;
+  DBusMessageIter appender, array_appender;
+  dbus_uint32_t zero = 0;
+  dbus_uint32_t invalid_flags = G_MAXUINT32;
+  const char *s;
+
+  if (f->address == NULL)
+    return;
+
+  dbus_connection_set_route_peer_messages (f->monitor, TRUE);
+
+  /* Try to become a monitor but specify nonzero flags - not allowed */
+
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_MONITORING, "BecomeMonitor");
+
+  if (m == NULL)
+    g_error ("OOM");
+
+  dbus_message_iter_init_append (m, &appender);
+
+  if (!dbus_message_iter_open_container (&appender, DBUS_TYPE_ARRAY, "s",
+        &array_appender))
+    g_error ("OOM");
+
+  if (!dbus_message_iter_close_container (&appender, &array_appender) ||
+      !dbus_message_iter_append_basic (&appender, DBUS_TYPE_UINT32,
+        &invalid_flags))
+    g_error ("OOM");
+
+  if (!dbus_connection_send_with_reply (f->monitor, m, &pc,
+        DBUS_TIMEOUT_USE_DEFAULT) ||
+      pc == NULL)
+    g_error ("OOM");
+
+  dbus_message_unref (m);
+  m = NULL;
+
+  if (dbus_pending_call_get_completed (pc))
+    test_pending_call_store_reply (pc, &m);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
+        &m, NULL))
+    g_error ("OOM");
+
+  while (m == NULL)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  g_assert_cmpint (dbus_message_get_type (m), ==, DBUS_MESSAGE_TYPE_ERROR);
+  g_assert_cmpstr (dbus_message_get_error_name (m), ==,
+      DBUS_ERROR_INVALID_ARGS);
+
+  /* Try to become a monitor but specify a bad match rule -
+   * also not allowed */
+
+  dbus_pending_call_unref (pc);
+  dbus_message_unref (m);
+
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_MONITORING, "BecomeMonitor");
+
+  if (m == NULL)
+    g_error ("OOM");
+
+  dbus_message_iter_init_append (m, &appender);
+
+  if (!dbus_message_iter_open_container (&appender, DBUS_TYPE_ARRAY, "s",
+        &array_appender))
+    g_error ("OOM");
+
+  /* Syntactically incorrect match rule taken from #92298 - was probably
+   * intended to be path='/modules/...'
+   */
+  s = "interface='org.kde.walletd',member='/modules/kwalletd/org.kde.KWallet/walletOpened'";
+
+  if (!dbus_message_iter_append_basic (&array_appender, DBUS_TYPE_STRING,
+        &s) ||
+      !dbus_message_iter_close_container (&appender, &array_appender) ||
+      !dbus_message_iter_append_basic (&appender, DBUS_TYPE_UINT32, &zero) ||
+      !dbus_connection_send_with_reply (f->monitor, m, &pc,
+        DBUS_TIMEOUT_USE_DEFAULT) ||
+      pc == NULL)
+    g_error ("OOM");
+
+  dbus_message_unref (m);
+  m = NULL;
+
+  if (dbus_pending_call_get_completed (pc))
+    test_pending_call_store_reply (pc, &m);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
+        &m, NULL))
+    g_error ("OOM");
+
+  while (m == NULL)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  g_assert_cmpint (dbus_message_get_type (m), ==, DBUS_MESSAGE_TYPE_ERROR);
+  g_assert_cmpstr (dbus_message_get_error_name (m), ==,
+      DBUS_ERROR_MATCH_RULE_INVALID);
+
+  dbus_pending_call_unref (pc);
+  dbus_message_unref (m);
+
+  /* We did not become a monitor, so we can still call methods. */
+
+  pc = NULL;
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS, "GetId");
+
+  if (m == NULL)
+    g_error ("OOM");
+
+  if (!dbus_connection_send_with_reply (f->monitor, m, &pc,
+        DBUS_TIMEOUT_USE_DEFAULT) ||
+      pc == NULL)
+    g_error ("OOM");
+
+  dbus_message_unref (m);
+  m = NULL;
+
+  if (dbus_pending_call_get_completed (pc))
+    test_pending_call_store_reply (pc, &m);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
+        &m, NULL))
+    g_error ("OOM");
+
+  while (m == NULL)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  ok = dbus_message_get_args (m, &f->e,
+      DBUS_TYPE_STRING, &s,
+      DBUS_TYPE_INVALID);
+  test_assert_no_error (&f->e);
+  g_assert (ok);
+  g_assert_cmpstr (s, !=, NULL);
+  g_assert_cmpstr (s, !=, "");
+
+  dbus_pending_call_unref (pc);
+  dbus_message_unref (m);
+}
+
 /*
  * Test the side-effects of becoming a monitor.
  */
@@ -541,7 +694,8 @@ test_become_monitor (Fixture *f,
 
   while (!got_unique || !got_a || !got_b || !got_c)
     {
-      test_main_context_iterate (f->ctx, TRUE);
+      if (g_queue_is_empty (&f->monitored))
+        test_main_context_iterate (f->ctx, TRUE);
 
       while ((m = g_queue_pop_head (&f->monitored)) != NULL)
         {
@@ -596,7 +750,8 @@ test_become_monitor (Fixture *f,
 
   while (!lost_unique || !lost_a || !lost_b || !lost_c)
     {
-      test_main_context_iterate (f->ctx, TRUE);
+      if (g_queue_is_empty (&f->monitored))
+        test_main_context_iterate (f->ctx, TRUE);
 
       while ((m = g_queue_pop_head (&f->monitored)) != NULL)
         {
@@ -651,7 +806,8 @@ test_become_monitor (Fixture *f,
 
   while (TRUE)
     {
-      test_main_context_iterate (f->ctx, TRUE);
+      if (g_queue_is_empty (&f->monitored))
+        test_main_context_iterate (f->ctx, TRUE);
 
       /* When we iterate all the connection's messages, we see ourselves
        * losing all our names, then we're disconnected. */
@@ -925,6 +1081,28 @@ test_method_call (Fixture *f,
 
   become_monitor (f);
 
+  /* regression test for
+   * https://bugs.freedesktop.org/show_bug.cgi?id=90952 */
+  m = dbus_message_new_method_call (f->recipient_name, "/foo",
+      DBUS_INTERFACE_PEER, "Ping");
+  dbus_connection_send (f->sender, m, NULL);
+  dbus_message_unref (m);
+
+  while (g_queue_get_length (&f->monitored) < 2)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  m = g_queue_pop_head (&f->monitored);
+  assert_method_call (m, f->sender_name, f->recipient_name, "/foo",
+      DBUS_INTERFACE_PEER, "Ping", "");
+  dbus_message_unref (m);
+
+  m = g_queue_pop_head (&f->monitored);
+  assert_method_reply (m, f->recipient_name, f->sender_name, "");
+  dbus_message_unref (m);
+
+  m = g_queue_pop_head (&f->monitored);
+  g_assert (m == NULL);
+
   m = dbus_message_new_method_call (f->recipient_name, "/foo", "com.example.bar",
       "Call1");
   dbus_connection_send (f->sender, m, NULL);
@@ -1131,6 +1309,8 @@ test_selective (Fixture *f,
   g_assert (m == NULL);
 }
 
+#ifdef DBUS_UNIX
+/* currently only used for the systemd activation test */
 static void
 expect_new_connection (Fixture *f)
 {
@@ -1157,6 +1337,7 @@ expect_new_connection (Fixture *f)
   dbus_message_unref (m);
 }
 
+/* currently only used for the systemd activation test */
 static void
 take_well_known_name (Fixture *f,
     DBusConnection *connection,
@@ -1170,6 +1351,7 @@ take_well_known_name (Fixture *f,
   g_assert_cmpint (ret, ==, DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER);
 }
 
+/* currently only used for the systemd activation test */
 static void
 expect_take_well_known_name (Fixture *f,
     DBusConnection *connection,
@@ -1399,6 +1581,7 @@ test_activation (Fixture *f,
       "com.example.Nope");
   dbus_message_unref (m);
 }
+#endif /* DBUS_UNIX */
 
 static void
 teardown (Fixture *f,
@@ -1463,6 +1646,8 @@ main (int argc,
 {
   test_init (&argc, &argv);
 
+  g_test_add ("/monitor/invalid", Fixture, NULL,
+      setup, test_invalid, teardown);
   g_test_add ("/monitor/become", Fixture, &side_effects_config,
       setup, test_become_monitor, teardown);
   g_test_add ("/monitor/broadcast", Fixture, NULL,
@@ -1489,8 +1674,12 @@ main (int argc,
       setup, test_unicast_signal, teardown);
   g_test_add ("/monitor/no-eavesdrop", Fixture, &no_eavesdrop_config,
       setup, test_unicast_signal, teardown);
+
+#ifdef DBUS_UNIX
+  /* this relies on the systemd activation code path */
   g_test_add ("/monitor/activation", Fixture, &fake_systemd_config,
       setup, test_activation, teardown);
+#endif
 
   return g_test_run ();
 }
