@@ -48,53 +48,16 @@
 /* http://www.tcpdump.org/linktypes.html */
 #define LINKTYPE_DBUS 231
 
-#ifdef DBUS_WIN
-
-/* gettimeofday is not defined on windows */
-#define DBUS_SECONDS_SINCE_1601 11644473600LL
-#define DBUS_USEC_IN_SEC        1000000LL
-
-#ifdef DBUS_WINCE
-
-#ifndef _IOLBF
-#define _IOLBF 0x40
-#endif
-#ifndef _IONBF
-#define _IONBF 0x04
-#endif
-
-void
-GetSystemTimeAsFileTime (LPFILETIME ftp)
-{
-  SYSTEMTIME st;
-  GetSystemTime (&st);
-  SystemTimeToFileTime (&st, ftp);
-}
-#endif
-
-static int
-gettimeofday (struct timeval *__p,
-	      void *__t)
-{
-  union {
-      unsigned long long ns100; /*time since 1 Jan 1601 in 100ns units */
-      FILETIME           ft;
-    } now;
-
-  GetSystemTimeAsFileTime (&now.ft);
-  __p->tv_usec = (long) ((now.ns100 / 10LL) % DBUS_USEC_IN_SEC);
-  __p->tv_sec  = (long)(((now.ns100 / 10LL) / DBUS_SECONDS_SINCE_1601) - DBUS_SECONDS_SINCE_1601);
-
-  return 0;
-}
-#endif
-
 static DBusHandlerResult
 monitor_filter_func (DBusConnection     *connection,
-		     DBusMessage        *message,
-		     void               *user_data)
+                     DBusMessage        *message,
+                     void               *user_data)
 {
-  print_message (message, FALSE);
+  long sec = 0, usec = 0;
+
+  _dbus_get_real_time (&sec, &usec);
+
+  print_message (message, FALSE, sec, usec);
   
   if (dbus_message_is_signal (message,
                               DBUS_INTERFACE_LOCAL,
@@ -107,14 +70,6 @@ monitor_filter_func (DBusConnection     *connection,
   return DBUS_HANDLER_RESULT_HANDLED;
 }
 
-#ifdef __APPLE__
-#define PROFILE_TIMED_FORMAT "%s\t%lu\t%d"
-#elif defined(__NetBSD__)
-#include <inttypes.h>
-#define PROFILE_TIMED_FORMAT "%s\t%" PRId64 "\t%d"
-#else
-#define PROFILE_TIMED_FORMAT "%s\t%lu\t%lu"
-#endif
 #define TRAP_NULL_STRING(str) ((str) ? (str) : "<none>")
 
 typedef enum
@@ -130,23 +85,30 @@ typedef enum
 } ProfileAttributeFlags;
 
 static void
-profile_print_with_attrs (const char *type, DBusMessage *message,
-  struct timeval *t, ProfileAttributeFlags attrs)
+profile_print_headers (void)
 {
-  printf (PROFILE_TIMED_FORMAT, type, t->tv_sec, t->tv_usec);
+  printf ("#type\ttimestamp\tserial\tsender\tdestination\tpath\tinterface\tmember\n");
+  printf ("#\t\t\t\t\tin_reply_to\n");
+}
+
+static void
+profile_print_with_attrs (const char *type, DBusMessage *message,
+  long sec, long usec, ProfileAttributeFlags attrs)
+{
+  printf ("%s\t%ld.%06ld", type, sec, usec);
 
   if (attrs & PROFILE_ATTRIBUTE_FLAG_SERIAL)
     printf ("\t%u", dbus_message_get_serial (message));
 
-  if (attrs & PROFILE_ATTRIBUTE_FLAG_REPLY_SERIAL)
-    printf ("\t%u", dbus_message_get_reply_serial (message));
-
   if (attrs & PROFILE_ATTRIBUTE_FLAG_SENDER)
     printf ("\t%s", TRAP_NULL_STRING (dbus_message_get_sender (message)));
 
   if (attrs & PROFILE_ATTRIBUTE_FLAG_DESTINATION)
     printf ("\t%s", TRAP_NULL_STRING (dbus_message_get_destination (message)));
 
+  if (attrs & PROFILE_ATTRIBUTE_FLAG_REPLY_SERIAL)
+    printf ("\t%u", dbus_message_get_reply_serial (message));
+
   if (attrs & PROFILE_ATTRIBUTE_FLAG_PATH)
     printf ("\t%s", TRAP_NULL_STRING (dbus_message_get_path (message)));
 
@@ -165,53 +127,61 @@ profile_print_with_attrs (const char *type, DBusMessage *message,
 static void
 print_message_profile (DBusMessage *message)
 {
-  struct timeval t;
+  static dbus_bool_t first = TRUE;
+  long sec = 0, usec = 0;
 
-  if (gettimeofday (&t, NULL) < 0)
+  if (first)
     {
-      printf ("un\n");
-      return;
+      profile_print_headers ();
+      first = FALSE;
     }
 
+  _dbus_get_real_time (&sec, &usec);
+
   switch (dbus_message_get_type (message))
     {
       case DBUS_MESSAGE_TYPE_METHOD_CALL:
-	profile_print_with_attrs ("mc", message, &t,
-	  PROFILE_ATTRIBUTE_FLAG_SERIAL |
-	  PROFILE_ATTRIBUTE_FLAG_SENDER |
-	  PROFILE_ATTRIBUTE_FLAG_PATH |
-	  PROFILE_ATTRIBUTE_FLAG_INTERFACE |
-	  PROFILE_ATTRIBUTE_FLAG_MEMBER);
-	break;
+        profile_print_with_attrs ("mc", message, sec, usec,
+          PROFILE_ATTRIBUTE_FLAG_SERIAL |
+          PROFILE_ATTRIBUTE_FLAG_SENDER |
+          PROFILE_ATTRIBUTE_FLAG_DESTINATION |
+          PROFILE_ATTRIBUTE_FLAG_PATH |
+          PROFILE_ATTRIBUTE_FLAG_INTERFACE |
+          PROFILE_ATTRIBUTE_FLAG_MEMBER);
+        break;
       case DBUS_MESSAGE_TYPE_METHOD_RETURN:
-	profile_print_with_attrs ("mr", message, &t,
-	  PROFILE_ATTRIBUTE_FLAG_SERIAL |
-	  PROFILE_ATTRIBUTE_FLAG_DESTINATION |
-	  PROFILE_ATTRIBUTE_FLAG_REPLY_SERIAL);
-	break;
+        profile_print_with_attrs ("mr", message, sec, usec,
+          PROFILE_ATTRIBUTE_FLAG_SERIAL |
+          PROFILE_ATTRIBUTE_FLAG_SENDER |
+          PROFILE_ATTRIBUTE_FLAG_DESTINATION |
+          PROFILE_ATTRIBUTE_FLAG_REPLY_SERIAL);
+        break;
       case DBUS_MESSAGE_TYPE_ERROR:
-	profile_print_with_attrs ("err", message, &t,
-	  PROFILE_ATTRIBUTE_FLAG_SERIAL |
-	  PROFILE_ATTRIBUTE_FLAG_DESTINATION |
-	  PROFILE_ATTRIBUTE_FLAG_REPLY_SERIAL);
-	break;
+        profile_print_with_attrs ("err", message, sec, usec,
+          PROFILE_ATTRIBUTE_FLAG_SERIAL |
+          PROFILE_ATTRIBUTE_FLAG_SENDER |
+          PROFILE_ATTRIBUTE_FLAG_DESTINATION |
+          PROFILE_ATTRIBUTE_FLAG_REPLY_SERIAL);
+        break;
       case DBUS_MESSAGE_TYPE_SIGNAL:
-	profile_print_with_attrs ("sig", message, &t,
-	  PROFILE_ATTRIBUTE_FLAG_SERIAL |
-	  PROFILE_ATTRIBUTE_FLAG_PATH |
-	  PROFILE_ATTRIBUTE_FLAG_INTERFACE |
-	  PROFILE_ATTRIBUTE_FLAG_MEMBER);
-	break;
+        profile_print_with_attrs ("sig", message, sec, usec,
+          PROFILE_ATTRIBUTE_FLAG_SERIAL |
+          PROFILE_ATTRIBUTE_FLAG_SENDER |
+          PROFILE_ATTRIBUTE_FLAG_DESTINATION |
+          PROFILE_ATTRIBUTE_FLAG_PATH |
+          PROFILE_ATTRIBUTE_FLAG_INTERFACE |
+          PROFILE_ATTRIBUTE_FLAG_MEMBER);
+        break;
       default:
-	printf (PROFILE_TIMED_FORMAT "\n", "tun", t.tv_sec, t.tv_usec);
-	break;
+        printf ("%s\t%ld.%06ld", "tun", sec, usec);
+        break;
     }
 }
 
 static DBusHandlerResult
-profile_filter_func (DBusConnection	*connection,
-		     DBusMessage	*message,
-		     void		*user_data)
+profile_filter_func (DBusConnection     *connection,
+                     DBusMessage        *message,
+                     void               *user_data)
 {
   print_message_profile (message);
 
@@ -248,7 +218,7 @@ binary_filter_func (DBusConnection *connection,
     {
       case BINARY_MODE_PCAP:
           {
-            struct timeval t = { 0, 0 };
+            long tv_sec, tv_usec;
             /* seconds, microseconds, bytes captured (possibly truncated),
              * original length.
              * http://wiki.wireshark.org/Development/LibpcapFileFormat
@@ -258,15 +228,9 @@ binary_filter_func (DBusConnection *connection,
             /* If this gets padded then we'd need to write it out in pieces */
             _DBUS_STATIC_ASSERT (sizeof (header) == 16);
 
-            if (_DBUS_UNLIKELY (gettimeofday (&t, NULL) < 0))
-              {
-                /* I'm fairly sure this can't actually happen */
-                perror ("dbus-monitor: gettimeofday");
-                exit (1);
-              }
-
-            header[0] = t.tv_sec;
-            header[1] = t.tv_usec;
+            _dbus_get_real_time (&tv_sec, &tv_usec);
+            header[0] = tv_sec;
+            header[1] = tv_usec;
 
             if (!tool_write_all (STDOUT_FILENO, header, sizeof (header)))
               {
@@ -431,7 +395,7 @@ main (int argc, char *argv[])
             usage (argv[0], 1);
         }
       else if (!strcmp (arg, "--help"))
-	usage (argv[0], 0);
+        usage (argv[0], 0);
       else if (!strcmp (arg, "--monitor"))
         {
           filter_func = monitor_filter_func;
@@ -453,9 +417,9 @@ main (int argc, char *argv[])
           binary_mode = BINARY_MODE_PCAP;
         }
       else if (!strcmp (arg, "--"))
-	continue;
+        continue;
       else if (arg[0] == '-')
-	usage (argv[0], 1);
+        usage (argv[0], 1);
       else {
           unsigned int filter_len;
           numFilters++;
@@ -483,12 +447,12 @@ main (int argc, char *argv[])
       if (connection)
         {
           if (!dbus_bus_register (connection, &error))
-      	    {
+            {
               fprintf (stderr, "Failed to register connection to bus at %s: %s\n",
-      	               address, error.message);
+                       address, error.message);
               dbus_error_free (&error);
               exit (1);
-      	    }
+            }
         }
     }
   else
@@ -519,6 +483,11 @@ main (int argc, char *argv[])
       exit (1);
     }
 
+  /* Receive o.fd.Peer messages as normal messages, rather than having
+   * libdbus handle them internally, which is the wrong thing for
+   * a monitor */
+  dbus_connection_set_route_peer_messages (connection, TRUE);
+
   if (!dbus_connection_add_filter (connection, filter_func,
                                    _DBUS_INT_TO_POINTER (binary_mode), NULL))
     {
@@ -548,7 +517,7 @@ main (int argc, char *argv[])
               dbus_bus_add_match (connection, filters[i] + offset, &error);
             }
 
-	  if (dbus_error_is_set (&error))
+          if (dbus_error_is_set (&error))
             {
               fprintf (stderr, "Failed to setup match \"%s\": %s\n",
                        filters[i], error.message);
